/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SocketServerThread.java 11:55:50 AM ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.communication.jndi.agent;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import org.ow2.petals.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.communication.jndi.agent.msg.response.AckResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.CreateSubcontextResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.ExceptionResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.ListBindingsResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.ListResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.LookupLinkResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.LookupResponse;
import org.ow2.petals.communication.jndi.agent.msg.response.RegistryResponse;
import org.ow2.petals.util.LoggingUtil;

/**
 * This class is the socket server for the distributed JNDI directory. It
 * receives requests from the naming context implementation client.
 *
 * @author ddesjardins - eBMWebsourcing
 */
public class JNDIServerThread extends Thread {

    /**
     * Logger Utility
     */
    private LoggingUtil log;

    /**
     * JNDI Agent Service fractal component
     */
    private JNDIAgentServiceImpl jndiAgentService;

    /**
     * The server host
     */
    private String host;

    /**
     * The server port
     */
    private int port;

    /**
     * The socket used for incoming requests
     */
    private ServerSocket serverSocket;

    /**
     * Boolean to know if the server is running
     */
    protected boolean run;

    /**
     * Creates a new instance of {@link JNDIServerThread}
     *
     * @param jndiAgentService
     */
    public JNDIServerThread(JNDIAgentServiceImpl jndiAgentService,
        LoggingUtil log, String host, int port) {
        this.jndiAgentService = jndiAgentService;
        this.log = log;
        this.host = host;
        this.port = port;
    }

    @Override
    public void run() {
        log.call();
        try {
            serverSocket = new ServerSocket();
            serverSocket.bind(new InetSocketAddress(host, port));

            log.info("PEtALS JNDI server started on " + host + ":" + port);

            RegistryRequest request = null;
            RegistryResponse response = null;
            ObjectInputStream objectInputStream = null;
            ObjectOutputStream objectOutputStream = null;

            Socket socket = null;
            run = true;
            synchronized (this) {
                this.notify();
            }
            while (run) {
                socket = serverSocket.accept();

                // wait for request
                try {
                    objectInputStream = new ObjectInputStream(socket
                        .getInputStream());
                    request = ((RegistryRequest) (objectInputStream
                        .readObject()));
                } catch (IOException e) {
                    // we receive some crappy data
                    request = null;
                } catch (Exception e) {
                    if ((objectInputStream.available()) != 0) {
                        log.warning("Unable to read JNDI Request Stream");
                    }
                    request = null;
                }

                if (request != null) {
                    try {

                        // process the request
                        Object object = jndiAgentService
                            .processRequest(request);

                        // send the response
                        switch (request.getType()) {
                            /*
                             * BIND
                             */
                            case bind:
                                response = new AckResponse();
                                break;
                            /*
                             * CREATE SUBCONTEXT
                             */
                            case createSubcontext:
                                response = new CreateSubcontextResponse(
                                    (String) object);
                                break;
                            /*
                             * DESTROY SUBCONTEXT
                             */
                            case destroySubcontext:
                                response = new AckResponse();
                                break;
                            /*
                             * LIST
                             */
                            case list:
                                response = new ListResponse(
                                    (NameClassPair[]) object);
                                break;
                            /*
                             * LIST BINDINGS
                             */
                            case listBindings:
                                response = new ListBindingsResponse(
                                    (Binding[]) object);
                                break;
                            /*
                             * LOOKUP
                             */
                            case lookup:
                                response = new LookupResponse(object);
                                break;
                            /*
                             * LOOKUP LINK
                             */
                            case lookupLink:
                                response = new LookupLinkResponse(object);
                                break;
                            /*
                             * PING
                             */
                            case ping:
                                response = new AckResponse();
                                break;
                            /*
                             * REBIND
                             */
                            case rebind:
                                response = new AckResponse();
                                break;
                            /*
                             * RENAME
                             */
                            case rename:
                                response = new AckResponse();
                                break;
                            /*
                             * UNBIND
                             */
                            case unbind:
                                response = new AckResponse();
                                break;
                            default:
                                break;
                        }

                    } catch (NamingException e) {
                        log.error(e.getMessage(), e);
                        response = new ExceptionResponse(e);
                    }
                    // write response to socket
                    objectOutputStream = new ObjectOutputStream(socket
                        .getOutputStream());
                    objectOutputStream.writeObject(response);
                    objectOutputStream.flush();
                    objectOutputStream.close();
                }
            }
        } catch (SocketException e) {
            if (!"Socket closed".equals(e.getMessage())) {
                log.error(e.getMessage(), e);
            }
        } catch (IOException e) {
            log.error("Error while starting socket server", e);
            run = false;
            throw new RuntimeException("Error while starting socket server", e);
        }
    }

    /**
     * Shutdown the server socket
     *
     */
    public void shutdown() {
        log.start();
        run = false;
        // Close the server socket
        try {
            this.interrupt();
            serverSocket.close();
        } catch (Exception e) {
            log.error("Problem while closing server socket " + e);
        }
        log.end();
    }

    /**
     *
     * @return
     */
    public boolean isStarted() {
        return run;
    }

}
