/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.container.thread;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

import javax.jbi.JBIException;

/**
 * This class is the parent class for managing tasks to perform on Components,
 * Bootstrap and ServiceAssemblies. Tasks on those objects are performed in a
 * dedicated thread, preventing the main thread to be blocked. When a timeout
 * exceeds on an action, the task is considered as blocked, and an exception is
 * thrown
 *
 * @author alouis,Christophe Hamerling - EBM WebSourcing
 *
 */
public abstract class AbstractThread extends Thread {

    /**
     * Exception thrown when a task failed in its own thread. Retrieved by the
     * main thread
     */
    protected JBIException jbiException;

    /**
     * The blocking working queue is equal to a FIFO. The worker thread is
     * waiting for an element to be added in this queue by the caller thread.
     *
     * This queue is the synchronization object between caller thread and
     * execution thread. When an element is added by the caller thread, the
     * execution thread wakes up and do the task.
     */
    protected BlockingQueue<String> requestQueue;

    /**
     * The response queue used to wait for the task to complete. The caller
     * thread is blocked on this queue while the task is not complted by the
     * worker thread.
     *
     * This queue is the synchronization object between execution thread and
     * caller thread. When the execution thread put an object in this queue, the
     * caller thread wakes up.
     */
    protected BlockingQueue<String> responseQueue;

    /**
     * Shutdown thread action
     */
    public static final String SHUTDOWNTHREAD = "SHUTDOWNTHREAD";

    /**
     * Time before an action is considered as blocked Exception is thrown when
     * this time exceed
     */
    protected long timeout = 0L;

    /**
     * Creates a new instance of AbstractThread
     */
    public AbstractThread(long taskTimeout) {
        super();
        this.requestQueue = new ArrayBlockingQueue<String>(1);
        this.responseQueue = new ArrayBlockingQueue<String>(1);
        timeout = taskTimeout;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Runnable#run()
     */
    @Override
    public void run() {
        boolean run = true;
        while (run) {
            try {
                // waits until there is one element available in the request
                // queue
                String action = requestQueue.take();
                // if return false, stop the loop and so stop the current
                // thread
                run = doTask(action);

                // put element in response element wakes up the caller thread
                responseQueue.put(action);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return;
    }

    /**
     * Execute the task in the dedicated thread. A JBI exception can occur during
     * the task execution. In this case, the local jbiException must be filled
     * with this exception and is shared between client and thread.
     *
     * @return false if the current thread needs to be shutdowned
     */
    protected abstract boolean doTask(String action);

    /**
     * Execute the action. If the thread is not alive, throw an exception. On
     * the other cases, put the action in the FIFO and wait for the task to
     * complete. This method is the entry point of the task submission, multiple
     * threads will wait for other threads to complete.
     *
     * @param action
     */
    private Object executeSynchronizer = new Object();

    protected void execute(String action) throws JBIException {
        /*
         * The method body is synchronized instead of the method itself because
         * a deadlock can occurs in the ComponentLifeCycleThread if the
         * component initialization calls Object.wait(this).
         */
        synchronized (executeSynchronizer) {
            /* thread is alive test */
            if (!this.isAlive()) {
                throw new JBIException("Thread " + this.getName() + " is dead");
            }

            try {
                /*
                 * put action in request FIFO, this will wait for an available
                 * place in the queue. Since the queue is one element size,
                 * block if there is a current task execution.
                 */
                requestQueue.put(action);

                /*
                 * wait response == wait end of execution of the current query,
                 * waiting for the doTask method to complete to be sure that
                 * tasks are not done simultaneously. if timeout exceed (the
                 * underlying operation is considered as blocked), throw an
                 * exception and kill the thread.
                 */
                String actionPerformed = null;
                if (timeout > 0L) {
                    actionPerformed = responseQueue.poll(timeout,
                        TimeUnit.MILLISECONDS);
                } else {
                    actionPerformed = responseQueue.take();
                }
                if (actionPerformed == null) {
                    this.interrupt();
                    throw new JBIException("Timeout exceeded when executing "
                        + action
                        + " management task. Assuming the task is blocked.");
                }

                /*
                 * Check if the underlying task executed in the dedicated thread
                 * thrown an exception; throws it if it exists.
                 */
                if (jbiException != null) {
                    JBIException e = jbiException;
                    jbiException = null;
                    throw e;
                }
            } catch (InterruptedException e1) {
                throw new JBIException("Interrupted exception during " + action
                    + " task execution.", e1);
            }
        }
    }

    /**
     * Shutdown the current thread
     *
     */
    public void shutdownThread() throws JBIException {
        execute(SHUTDOWNTHREAD);
    }
}
