/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentContextImpl.java 6254 2007-12-13 10:49:31Z rnaudin $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.component.context;

import static org.ow2.petals.container.ContainerService.COMPONENT_LOGGER_PREFIX;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.Set;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.api.MonologFactory;
import org.objectweb.util.monolog.api.TopicalLogger;
import org.ow2.petals.container.lifecycle.ComponentContextCommunication;
import org.ow2.petals.container.lifecycle.ComponentLifeCycle;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.ow2.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.ow2.petals.jbi.messaging.endpoint.InternalEndpoint;
import org.ow2.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.util.LoggingUtil;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * @version $Revision: 6254 $ $Date: 2006-08-04 12:11:16 +0200 (ven., 04 août
 *          2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author wjoseph - eBMWebsourcing
 */
public class ComponentContextImpl implements javax.jbi.component.ComponentContext {

    protected DeliveryChannelImpl deliveryChannel;

    /**
     * references of registered external serviceEndpoints
     */
    protected Set<ServiceEndpoint> externalEndpoints;

    /**
     * references of registered internal serviceEndpoints
     */
    protected Set<AbstractEndpoint> internalEndpoints;

    protected Component jbiComponent;

    /**
     * The Monolog loggerFactory
     */
    private MonologFactory loggerFactory;

    /**
     * The component context communication (can be either an installer or a
     * component life cycle)
     */
    private org.objectweb.fractal.api.Component comp;

    protected LoggingUtil logger;

    /**
     * Constructor of the ComponentContextImpl class
     * 
     * @param comp
     *            the fractal representation of installer or component life
     *            cycle component
     * @throws IllegalArgumentException
     *             Fractal Error: Impossible to get the service interface
     */
    public ComponentContextImpl(org.objectweb.fractal.api.Component comp) {
        super();

        this.comp = comp;
        org.objectweb.util.monolog.api.Logger log = getLogger();
        this.logger = new LoggingUtil(log, log.getName().substring(
                log.getName().lastIndexOf(".") + 1));

        this.internalEndpoints = new HashSet<AbstractEndpoint>();
        this.externalEndpoints = new HashSet<ServiceEndpoint>();
    }

    /**
     * Accessor to ComponentContextCommunication. Must be used to retrieve the
     * ComponentContextCommunication object through Fractal mechanism
     * 
     * @throws IllegalArgumentException
     *             Fractal Error: Impossible to get the service interface
     */
    protected ComponentContextCommunication getComponentContextCommunication() {
        try {
            return (ComponentContextCommunication) comp.getFcInterface("/content");
        } catch (NoSuchInterfaceException e) {
            throw new IllegalArgumentException(
                    "Fractal Error: Impossible to get the service interface", e);
        }
    }

    /**
     * @see javax.jbi.component.ComponentContext#activateEndpoint(javax.xml.namespace.QName,
     *      java.lang.String)
     */
    public ServiceEndpoint activateEndpoint(QName serviceName, String endpointName)
            throws JBIException {
        logger.start();
        if (serviceName == null) {
            throw new IllegalArgumentException("serviceName must be non-null");
        }
        if (endpointName == null) {
            throw new IllegalArgumentException("endpointName must be non-null");
        }
        InternalEndpoint se = getComponentContextCommunication().getEndpointService()
                .activateEndpoint(serviceName, endpointName,
                        getComponentContextCommunication().getAddress());
        internalEndpoints.add(se);
        logger.end();
        return se;
    }

    /**
     * @see javax.jbi.component.ComponentContext#deactivateEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void deactivateEndpoint(ServiceEndpoint endpoint) throws JBIException {
        logger.start();
        if (endpoint == null) {
            throw new IllegalArgumentException("endpoint must be non-null");
        }
        getComponentContextCommunication().getEndpointService().deactivateEndpoint(endpoint);
        internalEndpoints.remove(endpoint);
        logger.end();
    }

    /**
     * deregister all endpoints (internal and external)
     * 
     */
    public void deregisterAllEndpoints() throws JBIException {
        logger.start();

        for (AbstractEndpoint se : internalEndpoints) {
            getComponentContextCommunication().getEndpointService().deactivateEndpoint(se);
        }
        internalEndpoints.clear();

        for (ServiceEndpoint se : externalEndpoints) {
            getComponentContextCommunication().getEndpointService().deregisterExternalEndpoint(se);
        }
        externalEndpoints.clear();

        logger.end();
    }

    /**
     * @see javax.jbi.component.ComponentContext#deregisterExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        logger.start();
        getComponentContextCommunication().getEndpointService().deregisterExternalEndpoint(
                externalEndpoint);
        externalEndpoints.remove(externalEndpoint);
        logger.end();
    }

    public ConsumerEndpoint getAddress() {
        logger.call();
        return getComponentContextCommunication().getAddress();
    }

    public Component getComponent() {
        return jbiComponent;
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getComponentName()
     */
    public String getComponentName() {
        return getComponentContextCommunication().getJBIDescriptor().getComponent()
                .getIdentification().getName();
    }

    /**
     * @see javax.jbi.component.ComponentContext#getDeliveryChannel()
     */
    public DeliveryChannel getDeliveryChannel() throws MessagingException {
        logger.start();
        if (deliveryChannel == null) {
            deliveryChannel = createDeliveryChannel();
        } else {
            if (deliveryChannel.isOpened()) {
                throw new MessagingException("The previous DeliveryChannel is still opened");
            } else {
                deliveryChannel = createDeliveryChannel();
            }
        }
        logger.end();
        return deliveryChannel;
    }

    /**
     * Return the actual deliveryChannel. Can be null.
     * 
     * @return
     */
    public DeliveryChannelImpl getDeliveryChannelImpl() {
        return deliveryChannel;
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getEndpoint(javax.xml.namespace.QName,
     *      java.lang.String)
     */
    public ServiceEndpoint getEndpoint(QName service, String name) {
        logger.start();
        if (service == null) {
            throw new IllegalArgumentException("service must not be null");
        }
        if (name == null) {
            throw new IllegalArgumentException("name must not be null");
        }
        ServiceEndpoint result = getComponentContextCommunication().getEndpointService()
                .getEndpoint(service, name, false);
        logger.end();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getEndpointDescriptor(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public Document getEndpointDescriptor(ServiceEndpoint endpoint) throws JBIException {
        logger.start();
        Document result = null;
        if (endpoint == null) {
            throw new IllegalArgumentException("endpoint must not be null");
        }
        if (getEndpoint(endpoint.getServiceName(), endpoint.getEndpointName()) != null) {
            /*
             * Checks if the endpoint exist and if it is activated
             */
            AbstractEndpoint abstractEndpoint = (AbstractEndpoint) endpoint;
            if (abstractEndpoint.getEndpointService() == null) {
                abstractEndpoint.setEndpointService(getComponentContextCommunication()
                        .getEndpointService());
            }
            result = abstractEndpoint.getEndpointDescriptor();
        }
        logger.end();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getEndpoints(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getEndpoints(QName interfaceName) {
        logger.start();
        ServiceEndpoint[] result = getComponentContextCommunication().getEndpointService()
                .getInternalEndpointsForInterface(interfaceName);
        logger.end();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getEndpointsForService(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getEndpointsForService(QName serviceName) {
        logger.start();
        ServiceEndpoint[] result = getComponentContextCommunication().getEndpointService()
                .getInternalEndpointsForService(serviceName);
        logger.end();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getExternalEndpoints(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpoints(QName interfaceName) {
        logger.start();
        ServiceEndpoint[] result = getComponentContextCommunication().getEndpointService()
                .getExternalEndpointsForInterface(interfaceName);
        logger.end();
        return result;
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getExternalEndpointsForService(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) {
        logger.start();
        ServiceEndpoint[] result = getComponentContextCommunication().getEndpointService()
                .getExternalEndpointsForService(serviceName);
        logger.end();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getInstallRoot()
     */
    public String getInstallRoot() {
        logger.call();
        String result = getComponentContextCommunication().getInstallationRoot();
        return result;
    }

    /**
     * @see javax.jbi.component.ComponentContext#getLogger(java.lang.String,
     *      java.lang.String)
     */
    public Logger getLogger(String suffix, String resourceBundleName)
            throws MissingResourceException, JBIException {
        logger.start();
        String loggerName = getComponentName();

        // append the suffix to the logger Name
        if (suffix == null) {
            throw new IllegalArgumentException("suffix must not be null");
        } else {
            if (suffix.length() > 0) {
                loggerName += "." + suffix;
            }
        }

        // get the corresponding logger
        Logger jdkLogger = getComponentLogger(loggerName, resourceBundleName);

        logger.end();
        return jdkLogger;
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getMBeanNames()
     */
    public MBeanNames getMBeanNames() {
        logger.call();
        return getComponentContextCommunication().getAdminService().getMBeanNames();
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getMBeanServer()
     */
    public MBeanServer getMBeanServer() {
        logger.call();
        return getComponentContextCommunication().getAdminService().getMBeanServer();
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getNamingContext()
     */
    public InitialContext getNamingContext() {
        logger.call();
        return getComponentContextCommunication().getInitialContext();
    }

    /**
     * TODO not implemented
     * 
     * @see javax.jbi.component.ComponentContext#getTransactionManager()
     */
    public Object getTransactionManager() {
        //CORRECTED FOR TCK
        //throw new NoSuchMethodError("not yet implemented in Petals");
        return null;
    }

    /**
     * @throws JBIException
     * @see javax.jbi.component.ComponentContext#getWorkspaceRoot()
     */
    public String getWorkspaceRoot() {
        logger.call();
        return getComponentContextCommunication().getWorkspaceRoot();
    }

    /**
     * @see javax.jbi.component.ComponentContext#registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        logger.start();
        if (externalEndpoint == null) {
            throw new IllegalArgumentException("externalEndpoint must not be null");
        }
        getComponentContextCommunication().getEndpointService().registerExternalEndpoint(
                externalEndpoint);
        externalEndpoints.add(externalEndpoint);
        logger.end();
    }

    // //////////////////////////////////////////////////////
    // Implementation specific
    // //////////////////////////////////////////////////////

    /**
     * TODO resolveEndpointReference : refactor
     * 
     * @see javax.jbi.component.ComponentContext#resolveEndpointReference(org.w3c.dom.DocumentFragment)
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) {
        logger.start();
        if (epr == null) {
            throw new IllegalArgumentException("epr must not be null");
        }
        ServiceEndpoint result = null;
        // Search for all engines components if one can resolve the EPR.
        Map<ObjectName, ComponentLifeCycle> engines = getComponentContextCommunication()
                .getAdminService().getEngineCompoLifeCycles();

        synchronized (engines) {
            for (Iterator<ComponentLifeCycle> iter = engines.values().iterator(); iter.hasNext()
                    && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                result = lifeCycle.getComponent().resolveEndpointReference(epr);
            }
        }

        if (result == null) {
            // Search for all binding components if one can resolve the EPR.
            Map<ObjectName, ComponentLifeCycle> bindings = getComponentContextCommunication()
                    .getAdminService().getBindingCompoLifeCycles();

            synchronized (bindings) {
                for (Iterator<ComponentLifeCycle> iter = bindings.values().iterator(); iter
                        .hasNext()
                        && result == null;) {
                    ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                    result = lifeCycle.getComponent().resolveEndpointReference(epr);
                }
            }
        }
        logger.end();
        return result;

    }

    public void setComponent(Component component) {
        this.jbiComponent = component;
    }

    public void setDeliveryChannel(DeliveryChannelImpl deliveryChannel) {
        this.deliveryChannel = deliveryChannel;
    }

    protected DeliveryChannelImpl createDeliveryChannel() {
        return new DeliveryChannelImpl(this);
    }

    public JBIDescriptor getJBIDescriptor() {
        return getComponentContextCommunication().getJBIDescriptor();
    }

    /**
     * The method creates or retrieve a logger for the component.
     * 
     * @param loggerName
     * @param resourceBundleName
     * 
     * @return the jdk form logger
     * 
     * @throws JBIException
     */
    protected Logger getComponentLogger(String loggerName, String resourceBundleName)
            throws JBIException {
        logger.start();

        Logger jdkLogger = null;

        // get a reference on the Monolog LoggerFactory
        if (loggerFactory == null) {
            loggerFactory = Monolog.getMonologFactory();
        }

        // get a new logger or retrieve it
        TopicalLogger newLogger = (TopicalLogger) loggerFactory.getLogger(
                COMPONENT_LOGGER_PREFIX + "." + loggerName, resourceBundleName);
        
        newLogger.setAdditivity(false);

        // if the logger already exits, return it directly
        if (newLogger.getHandler().length > 0) {
            jdkLogger = (Logger) newLogger;
            return jdkLogger;
        }        

        // retrieve the generic console handler for components
        Handler genericHandler = loggerFactory.getHandler("componentsConsole");

        // create the console handler for the new logger
        Handler componentHandler = loggerFactory.createHandler(getComponentName(), "console");

        // copy all the attributes from the generic handler to the new handler
        for (String attributeName : genericHandler.getAttributeNames()) {
            if (!attributeName.equals("handlertype")) {
                componentHandler.setAttribute(attributeName, genericHandler
                        .getAttribute(attributeName));
            }
        }

        // customize the pattern and activate the handler
        componentHandler.setAttribute(Handler.PATTERN_ATTRIBUTE, "[" + COMPONENT_LOGGER_PREFIX.toLowerCase()
                + "." + getComponentName() + "] %m%n");
        componentHandler.setAttribute("activation", loggerFactory);

        // add the handler to the new logger
        try {
            newLogger.addHandler(componentHandler);
            newLogger.addHandler(loggerFactory.getHandler("petalsLF"));
        } catch (Exception e) {
            throw new JBIException(e);
        }

        // cast the logger to fit the JDK Logger class
        jdkLogger = (Logger) newLogger;

        logger.end();
        return jdkLogger;
    }

    public Router getRouter() {
        return getComponentContextCommunication().getRouter();
    }

    public org.objectweb.util.monolog.api.Logger getLogger() {
        return getComponentContextCommunication().getLogger();
    }

}
