/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegisterAllConnectionTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.descriptor.Connection;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.endpoint.EndpointService;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task registers connections if exist <br/> Inputs : {@link JBIDescriptor}
 * <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class RegisterAllConnectionTask implements Task {

    protected EndpointService endpointService;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public RegisterAllConnectionTask(EndpointService endpointService, LoggingUtil log) {
        super();
        this.endpointService = endpointService;
        this.log = log;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context.get(TaskContextConstants.DESCRIPTOR);

        List<Connection> connections = descriptor.getServiceAssembly().getConnections();
        List<Connection> successfulyRegisteredConnections = new ArrayList<Connection>();

        if (connections != null) {
            for (Connection connection : connections) {
                try {
                    registerConnection(connection, descriptor);
                } catch (ManagementException e) {
                    // Unregister previously registered connection
                    for (Connection successConnection : successfulyRegisteredConnections) {
                        try {
                            removeConnection(successConnection);
                        } catch (Exception e1) {
                            log.error("Can't delete an already registered connection");
                        }
                    }
                    throw e;
                }
                successfulyRegisteredConnections.add(connection);
            }
        }
    }

    /**
     * Register a connection into the endpoint registry of the JBI container
     * 
     * @param connection
     *            The {@link Connection} to register
     * @param descriptor
     *            the service assembly descriptor
     * @throws ManagementException
     *             when container fails to register a connection into the
     *             endpoint registry
     */
    protected void registerConnection(Connection connection, JBIDescriptor descriptor)
            throws ManagementException {
        String saName = DeploymentUtils.getServiceAssemblyName(descriptor);

        QName consInterface = connection.getConsumerInterfaceName();
        QName consService = connection.getConsumerServiceName();
        String consEndpoint = connection.getConsumerEndpointName();
        QName provService = connection.getProviderServiceName();
        String provEndpoint = connection.getProviderEndpointName();

        try {
            if (consInterface != null) {

                endpointService.createConnection(consInterface, provService, provEndpoint);

            } else {
                endpointService.createConnection(consService, consEndpoint, provService,
                        provEndpoint);
            }
        } catch (JBIException e) {
            String msg = "Error while registering a service unit connection for the service assembly: "
                    + saName + ".";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        JBIDescriptor descriptor = (JBIDescriptor) context.get(TaskContextConstants.DESCRIPTOR);

        List<Connection> connections = descriptor.getServiceAssembly().getConnections();
        if (connections != null) {
            for (Connection connection : connections) {
                try {
                    removeConnection(connection);
                } catch (Exception e) {
                    String msg = "Failed to revert a RegisterAllConnectionTask";
                    log.error(msg, e);
                }
            }
        }
    }

    /**
     * Remove a connection registered during the SA deployment process
     * 
     * @param connection
     *            a Connection to remove
     * @throws Exception
     *             if problem occurs during the deletion of a specific
     *             connection
     */
    protected void removeConnection(Connection connection) throws Exception {
        QName consInterface = connection.getConsumerInterfaceName();
        QName consService = connection.getConsumerServiceName();
        String consEndpoint = connection.getConsumerEndpointName();
        QName provService = connection.getProviderServiceName();
        String provEndpoint = connection.getProviderEndpointName();

        if (provEndpoint != null && provService != null) {
            if (consInterface != null) {
                endpointService.deleteConnection(consInterface, provService, provEndpoint);
            } else if (consService != null && consEndpoint != null) {
                endpointService.deleteConnection(consService, consEndpoint, provService,
                        provEndpoint);
            }
        }
    }

}
