/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: UndeployAllSUsTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.deployment.undeploy;

import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.management.LifeCycleMBean;

import org.ow2.petals.container.lifecycle.ComponentLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.jbi.management.util.XMLResult;
import org.ow2.petals.processor.Task;

/**
 * This task executes the undeployment process of a service unit, for each
 * Service Unit included into the service assembly <br/> Inputs :
 * {@link ServiceAssemblyLifeCycle}, {@link XMLResult}
 *
 * @author ofabre - EBM Websourcing
 *
 */
public class UndeployAllSUsTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminServiceMBean managerService;

    public UndeployAllSUsTask(AdminServiceMBean managerService) {
        super();
        this.managerService = managerService;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
            .get(DeploymentContextConstants.SA_LIFECYCLE);

        XMLResult xmlResult = (XMLResult) context
            .get(DeploymentContextConstants.XML_RESULT);

        /*
         * Undeploy sus
         */
         undeploySUS(saLifeCycle, xmlResult);

    }

    /**
     * Undeploy all SUs by calling the "undeploySU" method of the corresponding
     * SUDeploymentManager.
     *
     * @param saLifeCycle
     *            the parent {@link ServiceAssemblyLifeCycle}
     * @param xmlResult
     *            xml task result
     * @throws Exception
     *             if an error occurs during the undeployment of a service unit
     */
    protected void undeploySUS(ServiceAssemblyLifeCycle saLifeCycle,
        XMLResult xmlResult) throws Exception {
        /*
         * Put the service assembly in the shutdown state
         */
        String saState = saLifeCycle.getCurrentState();
        if (LifeCycleMBean.SHUTDOWN.equals(saState) || LifeCycleMBean.UNKNOWN.equals(saState)) {
            /*
             * The service assembly is in the SHUTDOWN state We can try to
             * undeploy it
             */

            /*
             * JBI spec specifies that service units must be undeployed in the
             * reverse order of the declared list in service assembly descriptor
             */
            List<ServiceUnit> suList = saLifeCycle.getServiceAssembly()
                .getServiceUnits();
            Collections.reverse(suList);
            StringBuffer componentOutput = new StringBuffer();
            for (ServiceUnit suDesc : suList) {
                String xmlReturn = undeploySU(saLifeCycle, suDesc);
                componentOutput.append(xmlReturn);
            }
            xmlResult.addComponentTaskResult(componentOutput);
        } else {
            throw new JBIException(
                "Service Assembly can't be undeployed in this state: "
                    + saState);
        }
    }

    /**
     * Process the undeployment of the related service unit.
     *
     * @param saLifeCycle
     *            the service assembly life cycle
     * @param serviceUnit
     *            the service unit production element
     * @return a {@link String} xml result of the service unit undeployment (see
     *         section 6.9 of the JSR 208)
     * @throws Exception
     *             if an error occurs during the undeployment process : can't
     *             retrieve the {@link ComponentLifeCycle}, can't retrieve the
     *             component {@link ServiceUnitManager} (SUM) or an exception is
     *             thrown during the call of the undeploy method of the SUM
     */
    protected String undeploySU(ServiceAssemblyLifeCycle saLifeCycle,
        ServiceUnit serviceUnit) throws Exception {
        String msg = "";
        String xmlReturn = null;

        /*
         * Undeploy it
         */
        String suName = DeploymentUtils.getServiceUnitName(serviceUnit);
        ServiceUnitLifeCycle serviceUnitLifeCycle = saLifeCycle
            .getServiceUnitsLifeCycles().get(suName);
        String targetedComponent = serviceUnit.getTargetComponentName();
        // Retrieve the component life cycle
        ComponentLifeCycle clc = managerService
            .getComponentLifeCycleByName(targetedComponent);
        if (clc == null) {
            msg = "The component (" + targetedComponent
                + ") is not deployed. The service unit cannot be undeployed.";
            throw new Exception(msg);
        }
        // Retrieve the service unit manager of the component
        ServiceUnitManager sum = clc.getComponent().getServiceUnitManager();
        if (sum == null) {
            msg = "The service unit manager is null for the component ("
                + targetedComponent
                + "). You may need to initialise the component. The service units cannot be undeployed.";
            throw new Exception(msg);
        }

        try {
            xmlReturn = sum.undeploy(suName, serviceUnitLifeCycle
                .getServiceUnitRootPath());
        } catch (DeploymentException e) {
            msg = "Service unit manager of the component (" + targetedComponent
                + ") failed to undeploy su (" + suName + ")";
            throw new Exception(msg, e);
        }

        /*
         * Remove the service unit from the service assembly
         */
        saLifeCycle.getServiceUnitsLifeCycles().remove(suName);

        return xmlReturn;
    }

    /**
     * Check if a component task result is valid against an xml schema
     *
     * @param componentTask
     *            xml of the component task
     * @param xsd
     *            an {@link InputStream} of the xsd file
     * @return true if valid, false if not valid
     */
    protected boolean isValidComponentTask(String componentTask, InputStream xsd) {
        return XMLResult.isValidComponentTask(componentTask, xsd);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // Nothing to do

    }

}
