/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.xml.bind.JAXBException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.topology.TopologyBuilder;
import org.objectweb.petals.topology.TopologyException;
import org.objectweb.petals.topology.generated.Container;
import org.objectweb.petals.topology.generated.DomainMode;
import org.objectweb.petals.topology.generated.Topology;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jndi.agent.JNDIAgentService;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.kernel.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.util.LoggingUtil;

/**
 * Class implementing the {@link ConfigurationService} interface.
 * 
 * @author Roland NAUDIN - EBMWebsourcing
 * @since Petals 2.0
 */

@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class))
public class ConfigurationServiceImpl implements ConfigurationService {

    /**
     * Logging utilities
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    private ContainerConfiguration containerConfiguration;

    private DomainConfiguration domainConfiguration;

    private Properties serverProperties;

    private Topology topology;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getContainerConfiguration()
     */
    public ContainerConfiguration getContainerConfiguration() {
        return this.containerConfiguration;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getDomainConfiguration()
     */
    public DomainConfiguration getDomainConfiguration() {
        return this.domainConfiguration;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getStaticTopology()
     */
    public Topology getStaticTopology() throws ConfigurationException {
        if (this.domainConfiguration.getMode().equals(DomainConfiguration.DomainMode.DYNAMIC)) {
            throw new ConfigurationException("The domain is not in static mode");
        }
        return topology;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#setContainerState(org.ow2.petals.kernel.configuration.ContainerConfiguration.ContainerState)
     */
    public void setContainerState(ContainerState state) {
        this.containerConfiguration.setState(state);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#verifyContainerTopology(org.ow2.petals.kernel.configuration.ContainerConfiguration,
     *      org.ow2.petals.kernel.configuration.DomainConfiguration)
     */
    public void verifyContainerTopology(ContainerConfiguration newContainerConfiguration,
            DomainConfiguration newDomainConfiguration) throws TopologyException {
        log.start();

        verifyDomain(newDomainConfiguration);

        verifyContainer(newContainerConfiguration);

        log.end();
    }

    /**
     * Start the fractal component.
     * 
     * @throws ConfigurationException
     * 
     * @throws JAXBException
     *             The parsing of the Topology file has failed
     * @throws IOException
     *             A configuration resource cannot be reached or read
     * @throws TopologyException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws ConfigurationException {
        log = new LoggingUtil(logger);
        log.start();

        try {
            // retrieve resources
            URL serverPropsURL = this.getClass().getResource(SERVER_PROPS_FILE);
            if (serverPropsURL == null) {

                throw new IOException("Failed to reach the resource '" + SERVER_PROPS_FILE + "'");
            }
            File serverPropsFile = new File(serverPropsURL.getPath());
            URL topologyURL = this.getClass().getResource(TOPOLOGY_FILE);
            if (topologyURL == null) {
                throw new IOException("Failed to reach the resource '" + TOPOLOGY_FILE + "'");
            }
            File topologyFile = new File(topologyURL.getPath());

            containerConfiguration = new ContainerConfiguration();
            containerConfiguration.init();
            domainConfiguration = new DomainConfiguration();

            createConfiguration(topologyFile, serverPropsFile);

        } catch (IOException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Configuration service", e);
            throw new ConfigurationException(e);
        } catch (ConfigurationException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Configuration service", e);
            throw e;
        } catch (JAXBException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Configuration service", e);
            throw new ConfigurationException(e);
        } catch (TopologyException e) {
            // log the error here as Fractal layer is removing the Exception
            // stack
            log.error("Failed to start Configuration service", e);
            throw new ConfigurationException(e);
        }

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.call();
    }

    /**
     * Create the container configuration based on the topology and server
     * files. Basic consistency is controlled between the 2 configuration files.
     * The {@code serverPropsFile} and <{@code topologyFile} fields must be non
     * null.
     * 
     * @param topologyFile
     *            The topology.xml file
     * @param serverPropsFile
     *            The server.properties file
     * 
     * @throws JAXBException
     * @throws TopologyException
     * @throws IOException
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    private void createConfiguration(File topologyFile, File serverPropsFile) throws JAXBException,
            TopologyException, IOException, ConfigurationException {
        log.start();

        // build the topology object
        TopologyBuilder topologyBuilder = TopologyBuilder.getInstance();
        topology = topologyBuilder.createTopology(topologyFile);

        // retrieve the server properties
        serverProperties = new Properties();
        serverProperties.load(new FileInputStream(serverPropsFile));

        String containerName = serverProperties
                .getProperty(ConfigurationService.PROPERTY_CONTAINER_NAME);
        if (containerName == null) {
            containerName = System.getProperty(PROPERTY_CONTAINER_NAME);
        }

        containerName = validateTopology(containerName);

        fillGlobalTopology();

        fillLocalTopology(containerName);

        fillGlobalProperties();

        log.end();
    }

    /**
     * Validate the consistency of the topology. The {@code topology} and the
     * {@code containerConfiguration} fields must be non {@code null}.
     * 
     * @param containerName
     *            The name of the local container retrieved from the
     *            environment, can be {@ null}
     * @throws ConfigurationException
     */
    private String validateTopology(String containerName) throws ConfigurationException {
        log.start();

        if (topology.getContainers().getContainer().size() == 0) {
            throw new ConfigurationException(
                    "A domain must contain at least one JBI container definition");
        }

        if (topology.getContainers().getContainer().size() != 1
                && !topology.getDomain().getMode().equals(DomainMode.STATIC)) {
            throw new ConfigurationException(
                    "Only static domain accepts mutiple JBI container definition");
        }

        // validate domain consistency
        if (DomainMode.STATIC.equals(topology.getDomain().getMode())) {
            if (containerName == null) {
                throw new ConfigurationException(
                        "The property 'petals.container.name' must be set in static distributed mode");
            }

            boolean isMatched = false;
            for (Container container : topology.getContainers().getContainer()) {
                if (container.getName().equals(containerName)) {
                    isMatched = true;
                    break;
                }
            }
            if (!isMatched) {
                throw new ConfigurationException(
                        "Cannot match the name of the local PEtALS container in the topology configuration");
            }

            if (DomainMode.STATIC.equals(topology.getDomain().getMode())) {
                validateStaticTopology();
            }
        } else {
            containerName = topology.getContainers().getContainer().get(0).getName();
        }

        // validate JNDI consistency
        if (!topology.getDomain().getJndi().getFactory().equals(
                JNDIAgentService.JNDI_PETALS_FACTORY)) {
            if (topology.getDomain().getJndi().getHost() == null
                    || topology.getDomain().getJndi().getPort() == null) {
                throw new ConfigurationException(
                        "JNDI host and port must be set for the external JNDI server with factory : '"
                                + topology.getDomain().getJndi().getFactory() + "'");
            }
        } else if ((topology.getDomain().getJndi().getPort() != null && topology.getDomain()
                .getJndi().getContainer() == null)
                || (topology.getDomain().getJndi().getPort() == null && topology.getDomain()
                        .getJndi().getContainer() != null)) {
            throw new ConfigurationException(
                    "JNDI container name and port must be set for to externalize the PEtALS JNDI server");
        }

        log.end();
        return containerName;
    }

    /**
     * Validate the consistency of the static topology
     * 
     * @throws ConfigurationException
     */
    private void validateStaticTopology() throws ConfigurationException {
        log.start();

        boolean jndiContainerHostFound = true;
        if (topology.getDomain().getJndi().getContainer() != null) {
            jndiContainerHostFound = false;
        }

        Collection<String> nameList = new HashSet<String>();
        Collection<Integer> joramIdList = new HashSet<Integer>();
        Map<String, HashSet<Integer>> portMap = new Hashtable<String, HashSet<Integer>>();

        for (Container container : topology.getContainers().getContainer()) {
            if (container.getHost() == null) {
                throw new ConfigurationException("The container named '" + container.getName()
                        + "' must declare its host element in static distributed mode");
            }

            if (!jndiContainerHostFound) {
                if (container.getName().equals(topology.getDomain().getJndi().getContainer())) {
                    jndiContainerHostFound = true;
                }
            }

            HashSet<Integer> portList;
            if (!portMap.containsKey(container.getHost())) {
                portList = new HashSet<Integer>();
                portMap.put(container.getHost(), portList);
            } else {
                portList = portMap.get(container.getHost());
            }
            if (!portList.add(container.getConfiguration().getNetworkservice().getPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getConfiguration().getNetworkservice().getPort() + " on host '"
                        + container.getHost() + "'");
            }
            if (container.getConfiguration().getJmxservice().getRmiport() != 0) {
                if (!portList.add(container.getConfiguration().getJmxservice().getRmiport())) {
                    throw new ConfigurationException("Duplicate port "
                            + container.getConfiguration().getJmxservice().getRmiport()
                            + " on host '" + container.getHost() + "'");
                }
            }
            if (container.getConfiguration().getJmxservice().getHttpport() != 0) {
                if (!portList.add(container.getConfiguration().getJmxservice().getHttpport())) {
                    throw new ConfigurationException("Duplicate port "
                            + container.getConfiguration().getJmxservice().getHttpport()
                            + " on host '" + container.getHost() + "'");
                }
            }
            if (!portList.add(container.getConfiguration().getJoramservice().getDomainPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getConfiguration().getJoramservice().getDomainPort()
                        + " on host '" + container.getHost() + "'");
            }
            if (!portList.add(container.getConfiguration().getJoramservice().getTcpPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getConfiguration().getJoramservice().getTcpPort()
                        + " on host '" + container.getHost() + "'");
            }
            if (!portList.add(container.getConfiguration().getDreamservice().getPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getConfiguration().getDreamservice().getPort() + " on host '"
                        + container.getHost() + "'");
            }
            if (!nameList.add(container.getName())) {
                throw new ConfigurationException("Duplicate container name '" + container.getName()
                        + "'");
            }
            if (!joramIdList.add(container.getConfiguration().getJoramservice().getId())) {
                throw new ConfigurationException("Duplicate Joram Id '"
                        + container.getConfiguration().getJoramservice().getId() + "'");
            }
        }

        if (!jndiContainerHostFound) {
            throw new ConfigurationException("The JNDI container host '"
                    + topology.getDomain().getJndi().getContainer() + "' is not defined");
        }

        log.end();
    }

    /**
     * Fill the global topology.
     */
    private void fillGlobalTopology() {
        log.start();

        domainConfiguration.setDomain(topology.getDomain().getName());
        if (DomainMode.STANDALONE.equals(topology.getDomain().getMode())) {
            domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.STANDALONE);
            // set the default value of the routing strategy of standalone mode
            containerConfiguration.setRoutingStrategy(Router.DEFAULT_STANDALONE_STRATEGY);
        } else if (DomainMode.STATIC.equals(topology.getDomain().getMode())) {
            domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.STATIC);
        } else {
            domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.DYNAMIC);
        }

        if (!DomainMode.STANDALONE.equals(topology.getDomain().getMode())) {
            domainConfiguration.setNetworkMuticastIp(topology.getDomain().getMulticast().getIp());
            domainConfiguration.setNetworkMuticastPort(topology.getDomain().getMulticast()
                    .getPort());
        }
        domainConfiguration.setJndiFactory(topology.getDomain().getJndi().getFactory());
        domainConfiguration.setJndiHost(topology.getDomain().getJndi().getHost());
        domainConfiguration.setJndiContainer(topology.getDomain().getJndi().getContainer());
        if (topology.getDomain().getJndi().getPort() != null) {
            domainConfiguration.setJndiPort(topology.getDomain().getJndi().getPort());
        }

        log.end();
    }

    /**
     * Fill the container configuration with its topology part. The
     * {@code containerConfiguration} and {@code topology} fields must be non
     * {@code null}.
     * 
     * @param container
     *            name The local container name
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    private void fillLocalTopology(String containerName) throws ConfigurationException {
        log.start();

        Container localContainer = null;
        for (Container container : topology.getContainers().getContainer()) {
            if (container.getName().equals(containerName)) {
                localContainer = container;
                break;
            }
        }

        if (localContainer == null) {
            throw new ConfigurationException(
                    "Failed to retrieve the container configuration for the container named '"
                            + containerName + "'");
        }

        containerConfiguration.setName(containerName);
        if (localContainer.getHost() != null) {
            containerConfiguration.setHost(localContainer.getHost());
        }
        containerConfiguration.setNetworkBindingPort(localContainer.getConfiguration()
                .getNetworkservice().getPort());
        containerConfiguration.setJmxLogin(localContainer.getConfiguration().getJmxservice()
                .getUser());
        containerConfiguration.setJmxPassword(localContainer.getConfiguration().getJmxservice()
                .getPassword());
        // can be null
        if (localContainer.getConfiguration().getJmxservice().getRmiport() != null) {
            containerConfiguration.setJmxRMIConnectorPort(localContainer.getConfiguration()
                    .getJmxservice().getRmiport());
        }
        // can be null
        if (localContainer.getConfiguration().getJmxservice().getHttpport() != null) {
            containerConfiguration.setJmxHTTPConnectorPort(localContainer.getConfiguration()
                    .getJmxservice().getHttpport());
        }
        containerConfiguration.setJoramId(localContainer.getConfiguration().getJoramservice()
                .getId());
        containerConfiguration.setJoramLogin(localContainer.getConfiguration().getJoramservice()
                .getUser());
        containerConfiguration.setJoramPassword(localContainer.getConfiguration().getJoramservice()
                .getPassword());
        containerConfiguration.setJoramDomainPort(localContainer.getConfiguration()
                .getJoramservice().getDomainPort());
        containerConfiguration.setJoramTCPPort(localContainer.getConfiguration().getJoramservice()
                .getTcpPort());
        containerConfiguration.setDreamTCPPort(localContainer.getConfiguration().getDreamservice()
                .getPort());

        log.end();
    }

    /**
     * Fill the container configuration with its global properties part. The
     * {@code containerConfiguration} and {@code serverProperties} fields must
     * be non {@code null}.
     */
    private void fillGlobalProperties() {
        log.start();

        for (Object key : serverProperties.keySet()) {
            if (key instanceof String) {
                if (((String) key).equals(PROPERTY_REPOSITORY_PATH)) {
                    containerConfiguration.setRepositoryDirectoryPath(serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_EXCHANGE_VALIDATION)) {
                    containerConfiguration.setExchangeValidation(new Boolean(serverProperties
                            .getProperty((String) key)));
                } else if (((String) key).equals(PROPERTY_TRANSPORT_QOS)) {
                    containerConfiguration.setTransportQOS(serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_ROUTING_STRATEGY)) {
                    containerConfiguration.setRoutingStrategy(serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_TASK_TIMEOUT)) {
                    containerConfiguration.setTaskTimeout(new Long(serverProperties
                            .getProperty((String) key)));
                }
            }
        }

        log.end();
    }

    /**
     * Verify the given new domain configuration against the topology.
     * 
     * @param newDomainConfiguration
     * @throws TopologyException
     */
    private void verifyDomain(DomainConfiguration newDomainConfiguration) throws TopologyException {
        if (!newDomainConfiguration.getMode().equals(domainConfiguration.getMode())) {
            throw new TopologyException("Domain mode mismatch");
        }

        // JNDI service
        if (!newDomainConfiguration.getJndiFactory().equals(domainConfiguration.getJndiFactory())) {
            throw new TopologyException("JNDI factory mismatch");
        }
        if (!(newDomainConfiguration.getJndiHost() == domainConfiguration.getJndiHost())) {
            if (newDomainConfiguration.getJndiHost() != null
                    && domainConfiguration.getJndiHost() != null) {
                if (!newDomainConfiguration.getJndiHost().equals(domainConfiguration.getJndiHost())) {
                    throw new TopologyException("JNDI Host mismatch");
                }
            }

        }
        if (!(newDomainConfiguration.getJndiContainer() == domainConfiguration.getJndiContainer())) {
            if (newDomainConfiguration.getJndiHost() != null
                    && domainConfiguration.getJndiHost() != null) {
                if (!newDomainConfiguration.getJndiHost().equals(domainConfiguration.getJndiHost())) {
                    throw new TopologyException("JNDI Host mismatch");
                }
            }
        }
        if (newDomainConfiguration.getJndiPort() != domainConfiguration.getJndiPort()) {
            throw new TopologyException("JNDI Port mismatch");
        }
    }

    /**
     * Verify the given new container configuration against the topology.
     * 
     * @param newContainerConfiguration
     * @throws TopologyException
     */
    private void verifyContainer(ContainerConfiguration newContainerConfiguration)
            throws TopologyException {

        if (DomainMode.STATIC.equals(topology.getDomain().getMode())) {
            Container newContainer = null;
            for (Container container : topology.getContainers().getContainer()) {
                if (container.getName().equals(newContainerConfiguration.getName())) {
                    newContainer = container;
                    break;
                }
            }

            if (newContainer == null) {
                throw new TopologyException("Container name unknown");
            }
            // network Service
            if (newContainer.getConfiguration().getNetworkservice().getPort() != newContainerConfiguration
                    .getNetworkBindingPort()) {
                throw new TopologyException("Network binding port mismatch");
            }
            // JMX Service
            if (!(newContainer.getConfiguration().getJmxservice().getUser() == newContainerConfiguration
                    .getJmxLogin())) {
                if (newContainer.getConfiguration().getJmxservice().getUser() != null
                        && newContainerConfiguration.getJmxLogin() != null) {
                    if (!newContainer.getConfiguration().getJmxservice().getUser().equals(
                            newContainerConfiguration.getJmxLogin())) {
                        throw new TopologyException("JMX user mismatch");
                    }
                }
            }
            if (!(newContainer.getConfiguration().getJmxservice().getPassword() == newContainerConfiguration
                    .getJmxPassword())) {
                if (newContainer.getConfiguration().getJmxservice().getPassword() != null
                        && newContainerConfiguration.getJmxPassword() != null) {
                    if (!newContainer.getConfiguration().getJmxservice().getPassword().equals(
                            newContainerConfiguration.getJmxPassword())) {
                        throw new TopologyException("JMX password mismatch");
                    }
                }
            }
            if (newContainer.getConfiguration().getJmxservice().getRmiport() != newContainerConfiguration
                    .getJmxRMIConnectorPort()) {
                throw new TopologyException("JMX RMI connector port mismatch");
            }
            if (newContainer.getConfiguration().getJmxservice().getHttpport() != newContainerConfiguration
                    .getJmxHTTPConnectorPort()) {
                throw new TopologyException("JMX HTTP connector port mismatch");
            }
            // JORAM service
            if (newContainer.getConfiguration().getJoramservice().getId() != newContainerConfiguration
                    .getJoramId()) {
                throw new TopologyException("JORAM Id mismatch");
            }
            if (!newContainer.getConfiguration().getJoramservice().getUser().equals(
                    newContainerConfiguration.getJoramLogin())) {
                throw new TopologyException("JORAM User mismatch");
            }
            if (!newContainer.getConfiguration().getJoramservice().getPassword().equals(
                    newContainerConfiguration.getJoramPassword())) {
                throw new TopologyException("JORAM password mismatch");
            }
            if (newContainer.getConfiguration().getJoramservice().getDomainPort() != newContainerConfiguration
                    .getJoramDomainPort()) {
                throw new TopologyException("JORAM Domain port mismatch");
            }
            if (newContainer.getConfiguration().getJoramservice().getTcpPort() != newContainerConfiguration
                    .getJoramTCPPort()) {
                throw new TopologyException("JORAM TCP port mismatch");
            }
            if (newContainer.getConfiguration().getDreamservice().getPort() != newContainerConfiguration
                    .getDreamTCPPort()) {
                throw new TopologyException("DREAM TCP port mismatch");
            }
        } else if (newContainerConfiguration.getHost().equals(containerConfiguration.getHost())) {
            if (newContainerConfiguration.getName() == containerConfiguration.getName()) {
                throw new TopologyException("Duplicate container name '"
                        + containerConfiguration.getName() + "'");
            }

        }
    }
}
