/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2007 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.ow2.petals.monitoring.router;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.NormalizedMessageImpl;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.1
 * @author alouis
 * 
 * This class contains all the states of a messageexchange lifecyle.
 */
public class ExchangeStateReport {

    private String id;

    private List<ExchangeStateCapture> reportList = new ArrayList<ExchangeStateCapture>();

    public ExchangeStateReport(MessageExchangeImpl exchange) {
        this.id = exchange.getExchangeId();

    }

    /**
     * transform a capture to a raw Map : "id" = <code>String</code>
     * exchangeId <br>
     * "time" = <code>long</code> capture.time <br>
     * "component" = <code>String</code> capture.component<br>
     * "role" = <code>String</code> capture.role<br>
     * "type" = <code>String</code> capture.messageType <br>
     * "status" = <code>String</code> capture.status <br>
     * 
     * @param capture
     * @return
     */
    protected static Map<String, Object> captureToRaw(String exchangeId,
        ExchangeStateCapture capture) {

        Map<String, Object> rawCapture = new HashMap<String, Object>();
        rawCapture.put("id", exchangeId);
        rawCapture.put("time", capture.time);
        rawCapture.put("component", capture.component);
        rawCapture.put("role", roleToRaw(capture.role));
        rawCapture.put("type", capture.messageType);
        rawCapture.put("status", statusToRaw(capture.status));
        rawCapture.put("content", capture.content);

        return rawCapture;
    }

    /**
     * 
     * @return
     * @throws TransformerConfigurationException
     */
    protected static Transformer getTransformer()
        throws TransformerConfigurationException {
        Transformer transformer = TransformerFactory.newInstance()
            .newTransformer();
        if (transformer == null) {
            transformer = TransformerFactory.newInstance().newTransformer();
            Properties props = new Properties();
            props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.setOutputProperties(props);
        }
        return transformer;
    }

    /**
     * Transform a report to a raw format :
     * <code>List(Map<String,Object>)</code> list of raw captures
     * 
     * @see captureToRaw(ExchangeStateCapture capture)
     * @param report
     * @return
     */
    protected static List<Map<String, Object>> reportToRaw(
        ExchangeStateReport report) {

        List<Map<String, Object>> rawCaptures = new ArrayList<Map<String, Object>>();
        for (ExchangeStateCapture capture : report.getReportList()) {
            rawCaptures.add(captureToRaw(report.getId(), capture));
        }
        return rawCaptures;
    }

    /**
     * Convert a Role to a string representation
     * 
     * @param role
     * @return "consumer" or "provider
     */
    protected static String roleToRaw(Role role) {
        return (Role.CONSUMER.equals(role)) ? "consumer" : "provider";
    }

    /**
     * Convert a status to a string representation
     * 
     * @param status
     * @return "active" or "done" or "error"
     */
    protected static String statusToRaw(ExchangeStatus status) {
        return (ExchangeStatus.DONE.equals(status)) ? "done"
            : (ExchangeStatus.ACTIVE.equals(status)) ? "active" : "error";
    }

    public String getId() {
        return id;
    }

    public List<ExchangeStateCapture> getReportList() {
        return reportList;
    }

    /**
     * Transforms a given MessageExchange into an ExchangeStateCapture
     * 
     * @param exchange
     *            the MessageExchangeImpl to convert
     * @param showMessageContent
     *            true if content must be set in the capture, false if not
     * @return
     */
    public ExchangeStateCapture messageToCapture(MessageExchangeImpl exchange,
        boolean showMessageContent) {
        ExchangeStateCapture state = new ExchangeStateCapture();
        state.time = System.currentTimeMillis();
        state.status = exchange.getStatus();
        state.role = exchange.getRole();

        if (Role.CONSUMER.equals(state.role)) {
            state.component = exchange.getConsumerEndpoint().getComponentName();
            state.container = exchange.getConsumerEndpoint().getContainerName();
        } else {
            state.component = ((AbstractEndpoint) exchange.getEndpoint())
                .getComponentName();
            state.container = ((AbstractEndpoint) exchange.getEndpoint())
                .getContainerName();
        }
        if (exchange.getFault() != null) {
            Fault fault = exchange.getFault();
            state.messageType = "fault";
            if (showMessageContent) {
                if (fault.getContent() != null) {
                    // FIXME ugly way to copy the fault content
                    try {
                        state.content = createString(fault.getContent());
                    } catch (Exception e) {
                        state.content = "<monitoring-error>Error occured "
                            + "while getting fault content</monitoring-error>";
                    }
                    try {
                        fault.setContent(createSource(state.content));
                    } catch (MessagingException e) {
                        e.printStackTrace();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            Map<String, NormalizedMessage> messages = exchange.getMessages();
            if (messages != null && messages.size() > 0) {
                NormalizedMessage message = new NormalizedMessageImpl();
                // searching for an "out" message
                // if not found -> taking first message
                if (messages.get("out") != null) {
                    message = messages.get("out");
                    state.messageType = "out";
                } else {
                    state.messageType = messages.keySet().iterator().next();
                    message = messages.get(state.messageType);
                }
                // adding message content and fault if wanted
                if (showMessageContent) {
                    if (message.getContent() != null) {
                        // FIXME ugly way to copy the message content
                        try {
                            state.content = createString(message.getContent());
                        } catch (Exception e) {
                            state.content = "<monitoring-error>Error occured "
                                + "while getting message content</monitoring-error>";
                        }
                        try {
                            message.setContent(createSource(state.content));
                        } catch (MessagingException e) {
                            e.printStackTrace();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        }
        return state;
    }

    /**
     * Transform a MessageExchange to a raw format :
     * <code>Map<String,Object></code> a raw map representation of the
     * message
     * 
     * @param exchangeId
     *            the id of the exchange containing the message
     * @param exchange
     *            the message exchange
     * @param showMessageContent
     *            true if content must be set in the message, false if not
     * @return a Map<String,Object> representation of the MessageExchange
     */
    public Map<String, Object> messageToRaw(String exchangeId,
        MessageExchangeImpl exchange, boolean showMessageContent) {
        return captureToRaw(exchangeId, messageToCapture(exchange,
            showMessageContent));
    }

    public void reportState(MessageExchangeImpl exchange,
        boolean showMessageContent) {
        reportList.add(messageToCapture(exchange, showMessageContent));
    }

    /**
     * Create a Source from the String. The string must be in UTF-8 format,
     * otherwise, the Source is not created.
     * 
     * @param msg
     *            an UTF-8 String
     * @return the Source built from the String, null if not an UTF-8
     */
    private Source createSource(final String msg) throws IOException {
        StreamSource source = new StreamSource();
        byte[] msgByte = msg.getBytes("UTF-8");
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setInputStream(in);
        return source;
    }

    /**
     * Create a String from a Source. The String will be in UTF-8.
     * 
     * @param s
     * @return
     * @throws Exception
     */
    private String createString(final Source s) throws Exception {

        String result = null;
        try {
        	if (s instanceof StreamSource
					&& ((StreamSource) s).getInputStream()
							.markSupported()) {
                StreamSource ss = (StreamSource) s;
                ss.getInputStream().reset();
                byte[] buf = new byte[ss.getInputStream().available()];
                BufferedInputStream bis = new BufferedInputStream(ss
                    .getInputStream());
                bis.read(buf);
                result = new String(buf);
            } else {
                StringWriter buffer = new StringWriter();
                Result sresult = new StreamResult(buffer);
                Transformer transform = getTransformer();
                transform.transform(s, sresult);
                result = buffer.toString();
            }
        } catch (IOException e) {
            throw new Exception(e);
        } catch (TransformerConfigurationException e) {
            throw new Exception(e);
        } catch (TransformerException e) {
            throw new Exception(e);
        }
        return result;
    }
}
