/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemStateServiceImpl.java 154 27 avr. 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.platform.systemstate;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.SystemUtil;
import org.ow2.petals.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Creates and reads state files for components, shared libs and service
 * assemblies. State files are text files that keep the state of JBI entities to
 * allow state retreval after Petals system shutdown or crash.
 *
 * @author Adrien LOUIS - EBM Websourcing
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.platform.systemstate.SystemStateService.class))
public class SystemStateServiceImpl implements SystemStateService {
    public static final String ARCHIVE_PATH_ATTRIBUTE = "archive-path";

    public static final String COMPONENT_ELEMENT = "component";

    public static final String COMPONENTS_ELEMENT = "components";

    public static final String INSTALL_PATH_ATTRIBUTE = "install-path";

    public static final String INSTALL_STATE_ATTRIBUTE = "install-state";

    public static final String LIFECYCLE_STATE_ATTRIBUTE = "lifecycle-state";

    public static final String NAME_ATTRIBUTE = "name";

    public static final String SERVICE_ASSEMBLIES_ELEMENT = "service-assemblies";

    public static final String SERVICE_ASSEMBLY_ELEMENT = "service-assembly";

    public static final String SHARE_LIBRARY_ELEMENT = "shared-library";

    public static final String SHARED_LIBRARIES_ELEMENT = "shared-libraries";

    public static final String SYSTEM_STATE_ELEMENT = "system-state";

    protected static final String NS = "http://org.objectweb.petals/xml/ns/system-state";

    protected static final String SCHEMA_FILE_PATH = "schema" + File.separator
        + "system-state.xsd";

    protected static final String STATE_FILE_PATH = "system-state.xml";

    protected Document document;

    protected File file;

    protected File schemaFile;

    @Requires(name = "repository", signature = org.ow2.petals.platform.repository.RepositoryService.class)
    protected RepositoryService repositorySrv;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    @Monolog(name = "logger")
    protected Logger logger;

    public SystemStateServiceImpl() {
    }

    public ComponentState createComponentStateHolder(String componentName,
        String installURL, String zipURL, String installState,
        String lifecycleState) throws Exception {
        Node e = createComponentNode(componentName, installURL, zipURL,
            installState, lifecycleState);

        getComponentsNode().appendChild(e);

        saveSystemState();

        return createComponent(e);
    }

    // ///////////////////////////////////////////////////////////////////////
    //
    // Document <-> File management methods
    //
    // ///////////////////////////////////////////////////////////////////////

    public ServiceAssemblyState createServiceAssemblyStateHolder(String saName,
        String installURL, String zipURL, String lifecycleState)
        throws Exception {
        Node e = createServiceAssemblyNode(saName, installURL, zipURL,
            lifecycleState);

        getServiceAssembliesNode().appendChild(e);

        saveSystemState();

        return createServiceAssembly(e);
    }

    public SharedLibraryState createSharedLibraryStateHolder(String slName,
        String installURL, String zipURL) throws Exception {
        Node e = createSharedLibraryNode(slName, installURL, zipURL);

        getSharedLibrariesNode().appendChild(e);

        saveSystemState();

        return createSharedLibrary(e);
    }

    public ComponentState deleteComponentStateHolder(String componentName)
        throws Exception {
        Node n = getComponentNode(componentName);

        deleteStateAndSave(getComponentsNode(), n);

        return createComponent(n);
    }

    public ServiceAssemblyState deleteServiceAssemblyStateHolder(String saName)
        throws Exception {
        Node n = getServiceAssemblyNode(saName);

        deleteStateAndSave(getServiceAssembliesNode(), n);

        return createServiceAssembly(n);
    }

    // ///////////////////////////////////////////////////////////////////////
    //
    // getNodeXXX methods
    //
    // ///////////////////////////////////////////////////////////////////////

    public SharedLibraryState deleteSharedLibraryStateHolder(String slName)
        throws Exception {
        Node n = getSharedLibraryNode(slName);

        deleteStateAndSave(getSharedLibrariesNode(), n);

        return createSharedLibrary(n);
    }

    /**
     * Initialize the SystemStateServiceImpl object. The recovery-file is
     * created or read, then this object is setup with the information contained
     * in the file.
     *
     * @throws IOException
     *             problems accessing the file
     * @throws SAXException
     *             the structure of the xml-file is not valid.
     */
    public void init() {
        try {
            loadStateStorageDocument();
        } catch (Exception e) {
            log.error("Can't load system-state file", e);
        }
        try {
            validateSystemStateXml();
        } catch (SAXException e) {
            log.error("Can't validate system-state file", e);
        } catch (IOException e) {
            log.error("Can't validate system-state file", e);
        }
    }

    public List<ComponentState> retrieveAllComponentStates() {
        NodeList elements = getComponentsNode().getChildNodes();

        List<ComponentState> list = new ArrayList<ComponentState>();

        for (int i = 0; i < elements.getLength(); i++) {
            list.add(createComponent(elements.item(i)));
        }
        return list;
    }

    public List<ServiceAssemblyState> retrieveAllServiceAssemblyStates() {
        NodeList elements = getServiceAssembliesNode().getChildNodes();

        List<ServiceAssemblyState> list = new ArrayList<ServiceAssemblyState>();

        for (int i = 0; i < elements.getLength(); i++) {
            list.add(createServiceAssembly(elements.item(i)));
        }
        return list;
    }

    public List<SharedLibraryState> retrieveAllSharedLibraryStates() {
        NodeList elements = getSharedLibrariesNode().getChildNodes();

        List<SharedLibraryState> list = new ArrayList<SharedLibraryState>();

        for (int i = 0; i < elements.getLength(); i++) {
            list.add(createSharedLibrary(elements.item(i)));
        }
        return list;
    }

    public void updateComponentInstallationState(String componentName,
        String installState) throws Exception {
        Node n = getComponentNode(componentName);

        updateStateAndSave(n, INSTALL_STATE_ATTRIBUTE, installState);
    }

    public void updateComponentLifeCycleState(String componentName,
        String lifecycleState) throws Exception {
        Node n = getComponentNode(componentName);

        updateStateAndSave(n, LIFECYCLE_STATE_ATTRIBUTE, lifecycleState);
    }

    public void updateServiceAssemblyState(String saName, String lifecycleState)
        throws Exception {
        Node n = getServiceAssemblyNode(saName);

        updateStateAndSave(n, LIFECYCLE_STATE_ATTRIBUTE, lifecycleState);
    }

    /**
     *
     * @param sourceURI
     * @return
     * @throws SAXException
     * @throws IOException
     */
    public void validateSystemStateXml() throws SAXException, IOException {
        // create a SchemaFactory capable of understanding WXS schemas
        SchemaFactory factory = SchemaFactory
            .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

        // load a WXS schema, represented by a Schema instance
        Source schemaSource = new StreamSource(schemaFile);
        Schema schema = factory.newSchema(schemaSource);

        // create a Validator instance which is be used to validate a document
        Validator validator = schema.newValidator();

        // validate
        validator.validate(new StreamSource(file));
    }

    /**
     * Create a ComponentState from the given node
     *
     * @param node
     * @return a ComponentState , null if node is null
     */
    protected ComponentState createComponent(Node node) {
        String name = XMLUtil.getAttributeValue(node, NAME_ATTRIBUTE);
        String iURL = XMLUtil.getAttributeValue(node, INSTALL_PATH_ATTRIBUTE);
        String aURL = XMLUtil.getAttributeValue(node, ARCHIVE_PATH_ATTRIBUTE);
        String iSte = XMLUtil.getAttributeValue(node, INSTALL_STATE_ATTRIBUTE);
        String lSte = XMLUtil
            .getAttributeValue(node, LIFECYCLE_STATE_ATTRIBUTE);

        return new ComponentState(name, iURL, aURL, iSte, lSte);
    }

    /**
     * create a Node representing a component state
     *
     * @param componentName
     * @param installURL
     * @param zipURL
     * @param installState
     * @param lifecycleState
     * @return
     * @throws Exception
     */
    protected Node createComponentNode(String componentName, String installURL,
        String zipURL, String installState, String lifecycleState)
        throws Exception {
        Node element = XMLUtil.createNode(document, COMPONENT_ELEMENT,
            NAME_ATTRIBUTE, componentName, INSTALL_PATH_ATTRIBUTE, installURL,
            ARCHIVE_PATH_ATTRIBUTE, zipURL, INSTALL_STATE_ATTRIBUTE,
            installState, LIFECYCLE_STATE_ATTRIBUTE, lifecycleState);

        return element;
    }

    /**
     * Create a ServiceAssemblyState from the given node
     *
     * @param node
     * @return a ServiceAssemblyState , null if node is null
     */
    protected ServiceAssemblyState createServiceAssembly(Node node) {
        String name = XMLUtil.getAttributeValue(node, NAME_ATTRIBUTE);
        String iURL = XMLUtil.getAttributeValue(node, INSTALL_PATH_ATTRIBUTE);
        String aURL = XMLUtil.getAttributeValue(node, ARCHIVE_PATH_ATTRIBUTE);
        String lSte = XMLUtil
            .getAttributeValue(node, LIFECYCLE_STATE_ATTRIBUTE);

        return new ServiceAssemblyState(name, iURL, aURL, lSte);
    }

    /**
     * create a Node representing a sa state
     *
     * @param saName
     * @param installURL
     * @param zipURL
     * @param lifecycleState
     * @return
     * @throws Exception
     */
    protected Node createServiceAssemblyNode(String saName, String installURL,
        String zipURL, String lifecycleState) throws Exception {
        Node element = XMLUtil.createNode(document, SERVICE_ASSEMBLY_ELEMENT,
            NAME_ATTRIBUTE, saName, INSTALL_PATH_ATTRIBUTE, installURL,
            ARCHIVE_PATH_ATTRIBUTE, zipURL, LIFECYCLE_STATE_ATTRIBUTE,
            lifecycleState);

        return element;
    }

    /**
     * Create a SharedLibraryState from the given node
     *
     * @param node
     * @return a SharedLibraryState , null if node is null
     */
    protected SharedLibraryState createSharedLibrary(Node node) {
        String name = XMLUtil.getAttributeValue(node, NAME_ATTRIBUTE);
        String iURL = XMLUtil.getAttributeValue(node, INSTALL_PATH_ATTRIBUTE);
        String aURL = XMLUtil.getAttributeValue(node, ARCHIVE_PATH_ATTRIBUTE);

        return new SharedLibraryState(name, iURL, aURL);
    }

    /**
     * create a Node representing a sl state
     *
     * @param slName
     * @param installURL
     * @param zipURL
     * @return
     * @throws Exception
     */
    protected Node createSharedLibraryNode(String slName, String installURL,
        String zipURL) throws Exception {

        Node element = XMLUtil.createNode(document, SHARE_LIBRARY_ELEMENT,
            NAME_ATTRIBUTE, slName, INSTALL_PATH_ATTRIBUTE, installURL,
            ARCHIVE_PATH_ATTRIBUTE, zipURL);

        return element;
    }

    // ///////////////////////////////////////////////////////////////////////
    //
    // service methods
    //
    // ///////////////////////////////////////////////////////////////////////

    /**
     * delete the element, and save the Document
     *
     * @param parent
     * @param child
     * @throws Exception
     *             node not found, saving problem
     */
    protected void deleteStateAndSave(Node parent, Node child) throws Exception {
        parent.removeChild(child);

        saveSystemState();
    }

    protected Node getComponentNode(String name) {
        return XMLUtil.getNodeWithAttribute(document, COMPONENT_ELEMENT,
            "name", name);
    }

    protected Node getComponentsNode() {
        return XMLUtil.getNode(document, COMPONENTS_ELEMENT);
    }

    protected Node getServiceAssembliesNode() {
        return XMLUtil.getNode(document, SERVICE_ASSEMBLIES_ELEMENT);
    }

    protected Node getServiceAssemblyNode(String name) {
        return XMLUtil.getNodeWithAttribute(document, SERVICE_ASSEMBLY_ELEMENT,
            "name", name);
    }

    protected Node getSharedLibrariesNode() {
        return XMLUtil.getNode(document, SHARED_LIBRARIES_ELEMENT);
    }

    protected Node getSharedLibraryNode(String name) {
        return XMLUtil.getNodeWithAttribute(document, SHARE_LIBRARY_ELEMENT,
            "name", name);
    }

    /**
     * add to the empty Document the necessary elements. clean the document
     * before if necessary
     *
     */
    protected void initializeDocument() {
        NodeList children = document.getChildNodes();

        for (int i = 0; i < children.getLength(); i++) {
            document.removeChild(children.item(i));
        }

        // document.setXmlVersion("1.0");

        Element s = document.createElement(SYSTEM_STATE_ELEMENT);
        Node c = document.createElement(COMPONENTS_ELEMENT);
        Node sl = document.createElement(SHARED_LIBRARIES_ELEMENT);
        Node sa = document.createElement(SERVICE_ASSEMBLIES_ELEMENT);

        s.setAttribute(XMLConstants.XMLNS_ATTRIBUTE, SystemStateServiceImpl.NS);
        document.appendChild(s);
        s.appendChild(c);
        s.appendChild(sl);
        s.appendChild(sa);
    }

    /**
     * Load the Document from the systemstate file. if the file is empty, the
     * document is initialized and saved
     */
    protected void loadStateStorageDocument() throws Exception {
        // create file if necessary
        if (!file.exists()) {
            file.createNewFile();
        }

        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
            .newInstance();

        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

        // if file is empty, initialize it (test the length of the file)
        if ((new FileInputStream(file)).available() == 0) {
            document = docBuilder.newDocument();
            initializeDocument();
            saveSystemState();
        } else {
            document = docBuilder.parse(file);
        }

    }

    /**
     * Write the document representing the system state to the recovery-file
     *
     * @throws Exception
     */
    protected void saveSystemState() throws Exception {
        Transformer transformer = TransformerFactory.newInstance()
            .newTransformer();

        FileWriter fileWriter = new FileWriter(file);
        transformer.transform(new DOMSource(document), new StreamResult(
            fileWriter));
    }

    /**
     * update the value of the attribute of the node, and save the Document
     *
     * @param node
     * @param att
     * @param value
     * @throws Exception
     *             node not found, saving problem
     */
    protected void updateStateAndSave(Node node, String att, String value)
        throws Exception {
        node.getAttributes().getNamedItem(att).setNodeValue(value);

        saveSystemState();
    }

    /**
     * Initialize the SystemStateService service
     *
     * @throws URISyntaxException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        log = new LoggingUtil(logger);
        log.start();

        File repoPath = repositorySrv.getRepositoryDirectory();
        File petalsPath = SystemUtil.getPetalsInstallDirectory();

        if (repoPath.exists()) {
            repoPath.mkdirs();
        }

        file = new File(repoPath.getAbsolutePath(), STATE_FILE_PATH);
        schemaFile = new File(petalsPath.getAbsolutePath(), SCHEMA_FILE_PATH);

        init();
        log.end();
    }

    public ComponentState getComponentState(String componentName) {
        return createComponent(getComponentNode(componentName));
    }

    public ServiceAssemblyState getServiceAssemblyState(String saName) {
        return createServiceAssembly(getServiceAssemblyNode(saName));
    }

    public SharedLibraryState getSharedLibraryState(String slName) {
        return createSharedLibrary(getSharedLibraryNode(slName));
    }

    public List<String> getAllDeployedServiceAssemblyNames() {
        List<String> names = new ArrayList<String>();
        for (ServiceAssemblyState state : retrieveAllServiceAssemblyStates()) {
            names.add(state.getName());
        }
        return names;
    }

    public List<String> getAllLoadedComponentNames() {
        List<String> names = new ArrayList<String>();
        for (ComponentState state : retrieveAllComponentStates()) {
            names.add(state.getName());
        }
        return names;
    }

    public List<String> getAllInstalledSharedLibNames() {
        List<String> names = new ArrayList<String>();
        for (SharedLibraryState state : retrieveAllSharedLibraryStates()) {
            names.add(state.getName());
        }
        return names;
    }

    public boolean isComponentLoaded(String componentName) {
        return getAllLoadedComponentNames().contains(componentName);
    }

    public boolean isSLLoaded(String slName) {
        return getAllInstalledSharedLibNames().contains(slName);
    }

    public boolean isSADeployed(String saName) {
        return getAllDeployedServiceAssemblyNames().contains(saName);
    }

}
