/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIUtil.java 16:03:11 ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.util;

import java.util.Map;
import java.util.Map.Entry;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.LinkRef;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * This utility class is used to help us to manage JNDI directories
 *
 * @author ddesjardins - eBMWebsourcing
 */
public final class JNDIUtil {

    private JNDIUtil() {

    }

    /**
     * Browse the JNDI server.
     *
     * @param initialContext
     *            The InitialContext connected to the JNDI server
     * @param host
     *            The JNDI host. Can be {@code null}
     * @param port
     *            The JNDI port
     * @return
     * @throws NamingException
     */
    public static String browseJNDI(InitialContext initialContext, String host,
        int port) throws NamingException {
        StringBuilder result = new StringBuilder();

        if (host == null) {
            result.append("JNDI directory local \n<== / ==>\n");
        } else {
            result.append("JNDI directory on " + host + ":" + port
                + "\n<== / ==>\n");
        }

        recurseBindings(initialContext, initialContext.listBindings("/"), "\t",
            result);

        return result.toString().replaceAll("&47;", "/");
    }

    /**
     * see isBound(Context parentContext,String childContextName, String name)
     *
     * @param parentContext
     * @param childContextName
     * @param name
     * @return
     * @throws NamingException
     */
    public static boolean isBound(Context parentContext, Name childContextName,
        Name name) throws NamingException {
        return isBound(parentContext, NamingHelper
            .nameToString(childContextName), NamingHelper.nameToString(name));
    }

    /**
     * see isBound(Context parentContext,String childContextName, String name)
     *
     * @param parentContext
     * @param childContextName
     * @param name
     * @return
     * @throws NamingException
     */
    public static boolean isBound(Context parentContext, Name childContextName,
        String name) throws NamingException {
        return isBound(parentContext, NamingHelper
            .nameToString(childContextName), name);
    }

    /**
     * see isBound(Context parentContext,String childContextName, String name)
     *
     * @param parentContext
     * @param childContextName
     * @param name
     * @return
     * @throws NamingException
     */
    public static boolean isBound(Context parentContext,
        String childContextName, Name name) throws NamingException {
        return isBound(parentContext, childContextName, NamingHelper
            .nameToString(name));
    }

    /**
     * Check if a name is bound is a specific context
     *
     * @param parentContext
     *            parent context where to list the content of the child context
     * @param childContextName
     *            name of the child context, the context in we want to look for
     *            the binding name
     * @param name
     *            binding name
     * @return
     * @throws NamingException
     */
    public static boolean isBound(Context parentContext,
        String childContextName, String name) throws NamingException {
        boolean test = false;
        NamingEnumeration<NameClassPair> namingEnumeration = parentContext
            .list(childContextName);
        while (namingEnumeration.hasMoreElements()) {
            NameClassPair classPair = namingEnumeration.next();
            if (classPair.getName().equals(name)) {
                return true;
            }
        }

        return test;
    }

    /**
     * Recurse the contexts
     *
     * @param bindings
     * @throws NamingException
     */
    protected static void recurseBindings(Context context,
        NamingEnumeration<Binding> bindings, String tabs, StringBuilder result)
        throws NamingException {

        while (bindings.hasMoreElements()) {

            Binding binding = bindings.next();

            Object object = binding.getObject();

            if (object instanceof Context) {

                result.append("\n" + tabs + "<=" + binding.getName() + "=>\n");

                try {

                    recurseBindings((Context) object, context
                        .listBindings(binding.getName()), tabs + "\t", result);
                } catch (Exception e) {

                    recurseNames((Context) object, context.list(binding
                        .getName()), tabs + "\t", result);
                }
            } else if (object instanceof Map) {

                result.append(tabs + "-> " + binding.getName() + "\n");

                Map<?, ?> map = (Map<?, ?>) object;

                for (Entry<?, ?> entry : map.entrySet()) {

                    result.append(tabs + "\t" + "-> (" + entry.getKey() + ") "
                        + entry.getValue() + "\n");
                }
            } else if (object instanceof LinkRef) {

                LinkRef linkRef = (LinkRef) binding.getObject();

                result.append(tabs + "-> " + binding.getName() + " [Link to "
                    + linkRef.getLinkName() + "]\n");

            } else {

                result.append(tabs + "-> " + binding.getObject() + "\n");
            }
        }
    }

    /**
     * Recurse the contexts
     *
     * @param bindings
     * @throws NamingException
     */
    protected static void recurseNames(Context context,
        NamingEnumeration<NameClassPair> names, String tabs,
        StringBuilder result) throws NamingException {

        while (names.hasMoreElements()) {

            NameClassPair name = names.next();

            result.append(tabs + "-> " + name.getName() + "\n");
        }
    }
}
